"""WARNING: Autogenerated by django-settings-model."""

DEBUG = True
SECRET_KEY = '2#mu)bp#5-2bru4n#yijvbp$isc=@9yvrtk_$cz2uvi7ziwd20'
TIME_ZONE = 'America/Chicago'
APPEND_SLASH = True
ALLOWED_HOSTS = ['*']



"""
Unit tests
"""

from django.contrib.auth.models import User
from django.test import Client, TestCase
from django.urls import reverse

from . import wsgi
from .models import Settings
from .settings import get_setting


class SettingsTestCase(TestCase):
    def test_bad_setting(self):
        self.assertIsNone(get_setting("INTENTIONALLY_BAD_VARIABLE"))


class SettingsModelTestCase(TestCase):
    """
    Test the example Settings implementation.
    """

    def setUp(self):
        Settings.init()
        self.settings = Settings.objects.filter(is_active=True).first()

    def test_constructor(self):
        self.assertTrue(self.settings)
        self.assertEqual(self.settings.name, "Default")

    def test_change_settings(self):
        old_debug = self.settings.debug_mode
        self.settings.debug_mode = not old_debug
        self.settings.save()
        self.assertEqual(self.settings.debug_mode, not old_debug)

    def test_write_and_reboot(self):
        self.settings.write_and_signal_reboot()


class SettingsModelAdminTestCase(TestCase):
    """
    Tests to ensure that the model admin returns an OK response.
    """

    def setUp(self):
        self.user = User.objects.create_superuser(
            username="test_user",
            password="test_password",
            email="test_email@example.org",
        )
        self.client = Client()
        self.login = self.client.login(username="test_user", password="test_password")

    def test_changelist_view(self):
        self.assertTrue(self.login)
        response = self.client.get(reverse("admin:settings_model_settings_changelist"))
        self.assertEqual(response.status_code, 200)

    def test_add_view(self):
        self.assertTrue(self.login)
        response = self.client.get(reverse("admin:settings_model_settings_add"))
        self.assertEqual(response.status_code, 200)


class WSGITestCase(TestCase):
    """
    Test WSGI module.
    """

    def test_wsgi_simple(self):
        self.assertTrue(wsgi.application)



from django.contrib import admin

from . import models


@admin.register(models.Settings)
class SettingsAdmin(admin.ModelAdmin):
    list_filter = ("is_active",)
    list_display = (
        ("name",)
        + list_filter
        + ("debug_mode", "secret_key", "time_zone", "append_slash", "allowed_hosts")
    )
    search_fields = list_display
    fieldsets = (
        (None, {"fields": ("name", "is_active")}),
        (
            "Settings",
            {
                "fields": (
                    "debug_mode",
                    "secret_key",
                    "time_zone",
                    "append_slash",
                    "allowed_hosts",
                )
            },
        ),
    )



import importlib
import logging
import os
from pathlib import Path
from pytz import common_timezones
import sys

from django.conf import settings
from django.db import models, transaction
from django.db.utils import Error as DBError
from django.utils.crypto import get_random_string
from django.utils.translation import gettext_lazy as _

from .settings import get_setting


logger = logging.getLogger(__name__)


class SettingsModel(models.Model):
    """
    An abstract model that tracks settings and touches certain files on save to signal
    the web server to reload. The ``FILE_NAME`` is a required property.
    """

    name = models.CharField(default="Default", max_length=255, unique=True, blank=False)
    is_active = models.BooleanField(default=True)

    # class constants, overridable by subclasses
    READABLE_INDEX = 2
    WRITABLE_INDEX = 3
    CREATE_INITIAL = True

    class Meta:
        abstract = True

    def __str__(self):
        return self.name

    @property
    def __settings_filename__(self):
        """
        The property defines the filename for the settings file which will be written to
        the project folder.
        """
        raise NotImplementedError

    @property
    def __settings_map__(self):
        """
        This property maps fields to the settings file variables.

        Returns a list/tuple of tuples in the format:
            (field_name, setting_name, readable, writable)

        For example:
            [
                ("debug_mode", "DEBUG", True, True),
                ("from_email", "DEFAULT_FROM_EMAIL", True, True),
            ]

        This should be implemented by subclasses with a normal list or tuple.
        """
        raise NotImplementedError

    def _get_settings_file(self):
        """
        The settings file should be saved next to the settings that are active. We will
        use the ``DJANGO_SETTINGS_MODULE`` environment variable to locate the directory
        where that file is located.

        The user may be using multiple settings models, so each one should define a
        unique ``__settings_filename__`` and we will write to that file name.
        """
        settings_file = importlib.import_module(
            os.environ["DJANGO_SETTINGS_MODULE"]
        ).__file__
        return os.path.join(os.path.dirname(settings_file), self.__settings_filename__)

    def encode_setting(self, field):
        """
        Given a field, convert the field to a string representing the Python code that
        should be written to the settings file.
        """
        raise NotImplementedError

    @classmethod
    def init(cls):
        """
        See if we have settings. If so, read the actual settings values into it.
        """
        logger.info("running settings init")
        try:
            s = cls.objects.filter(is_active=True).first()
            if s:
                s.read_settings()
            elif cls.CREATE_INITIAL:
                s = cls()
                s.read_settings()
        except DBError:
            logger.warning("db not ready (error on {} model)".format(cls.__name__))
            return

    def read_settings(self):
        """
        Read settings from Django into this instance.
        """
        for s in [x for x in self.__settings_map__ if x[self.READABLE_INDEX]]:
            try:
                setattr(self, s[0], getattr(settings, s[1]))
            except AttributeError:  # setting not found
                pass
        return self.save(reboot=False)

    def write_settings(self):
        """
        Write the settings defined by this instance to the settings file.
        """
        text = '"""WARNING: Autogenerated by django-settings-model."""\n\n'
        for s in [x for x in self.__settings_map__ if x[self.WRITABLE_INDEX]]:
            content = self.encode_setting(getattr(self, s[0]))
            if content:
                text += "{} = {}\n".format(s[1], content)

        # write to settings file
        with open(self._get_settings_file(), "w") as f:
            logger.info("writing config to {}".format(f.name))
            f.write(text)

    @transaction.atomic
    def save(self, *args, **kwargs):
        """
        Save this instance, check to ensure only one instance is active, and reboot, if
        necessary.
        """
        # extract custom kwargs
        reboot = kwargs.pop("reboot", True)

        # deactivate extra settings if needed
        actives = type(self).objects.filter(is_active=True).exclude(pk=self.pk)
        if self.is_active:
            actives.update(is_active=False)
        else:
            active = actives.first()
            actives.exclude(pk=active.pk).update(is_active=False)

        # if we are saving an active settings instance, reboot the web server
        if self.is_active:
            if reboot:
                transaction.on_commit(lambda: self.write_and_signal_reboot())
        return super().save(*args, **kwargs)

    @staticmethod
    def signal_reboot():
        """
        Find and touch the reboot files to signal the server to restart.
        """
        # get reboot files
        touch_files = get_setting("SETTINGS_MODEL_REBOOT_FILES")
        if not touch_files:
            touch_files = [
                os.path.join(get_setting("BASE_DIR"), "manage.py")
            ]  # dev server
            try:
                wsgi_module = get_setting("WSGI_APPLICATION").rsplit(".", 1)[0]
                try:
                    importlib.import_module(wsgi_module)
                except RuntimeError:
                    pass
                touch_files.append(sys.modules[wsgi_module].__file__)
            except (AttributeError, IndexError, KeyError, ImportError):
                pass

        # touch files to signal reboot
        logger.info("signalling webserver reboot")
        logger.debug("  touching:")
        for f in touch_files:
            p = Path(f)
            if p.exists():
                logger.debug("   - {}".format(f))
                p.touch()
            else:
                logger.debug("   - {} (skipped, doesn't exist)".format(f))

    def write_and_signal_reboot(self, commit=True):
        """
        Commit the configuration to disk and call `signal_reboot`.
        """
        if commit:
            logger.info("committing config to disk...")
            self.write_settings()

        # signal reboot
        self.signal_reboot()


class Settings(SettingsModel):
    """
    A basic implementation of a settings model.
    """

    debug_mode = models.BooleanField(default=True)
    secret_key = models.CharField(max_length=255, blank=True)
    tz_choices = [(x, x) for x in common_timezones]
    time_zone = models.CharField(max_length=255, blank=True, choices=tz_choices)
    append_slash = models.BooleanField(default=False)
    allowed_hosts = models.CharField(
        max_length=255,
        blank=True,
        help_text=_(
            "A comma-delimited list of hosts for which this application is allowed to "
            "respond to."
        ),
    )

    __settings_filename__ = "model_settings.py"

    __settings_map__ = [
        ("debug_mode", "DEBUG", True, True),
        ("secret_key", "SECRET_KEY", True, True),
        ("time_zone", "TIME_ZONE", True, True),
        ("append_slash", "APPEND_SLASH", True, True),
        ("allowed_hosts", "ALLOWED_HOSTS", True, True),
    ]

    class Meta:
        verbose_name = verbose_name_plural = "Settings"

    @classmethod
    def check_secret_key(cls):
        """
        Attempt to get the active settings, and if the secret key is set to the default
        one, then create a new randomized secret key.
        """
        logger.info("attempting to check the secret key...")
        try:
            s = cls.objects.filter(is_active=True).first()
            if s and s.secret_key == "not-a-very-good-secret":
                chars = "abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)"
                s.secret_key = get_random_string(50, chars)
                s.save()
        except DBError:
            logger.warning("db not ready (error on {} model)".format(cls.__name__))
            return

    def encode_setting(self, field):
        if field is self.allowed_hosts:
            return str(field)
        return field.__repr__()



from django.apps import AppConfig


class CustomConfig(AppConfig):
    name = "settings_model"
    verbose_name = "Settings"

    def ready(self, *args, **kwargs):
        # run settings initializer
        from .models import Settings

        Settings.init()
        Settings.check_secret_key()



"""
This module is designed to do 2 things:

 1. Provide an interface (``get_setting``) for getting settings from
    ``django.conf.settings`` with settings in this file being the fallback default.
 2. Provide enough settings for this app to run as a standalone project.
"""

from django.conf import settings
import os
import sys


def get_setting(name):
    """
    Hook for getting Django settings and using properties of this file as the default.
    """
    me = sys.modules[__name__]
    return getattr(settings, name, getattr(me, name, None))


# app-specific settings
SETTINGS_MODEL_REBOOT_FILES = []


# standalone settings
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = "not-a-very-good-secret"
DEBUG = True
ALLOWED_HOSTS = ["*"]
INSTALLED_APPS = [
    "settings_model",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]
ROOT_URLCONF = "settings_model.urls"
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]
WSGI_APPLICATION = "settings_model.wsgi.application"
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {"detail": {"format": "%(name)s [%(levelname)s]: %(message)s"}},
    "handlers": {"console": {"class": "logging.StreamHandler", "formatter": "detail"}},
    "loggers": {"": {"handlers": ["console"], "level": "DEBUG", "propagate": True}},
}
LANGUAGE_CODE = "en-us"
TIME_ZONE = "America/Chicago"
USE_I18N = True
USE_L10N = True
USE_TZ = True
STATIC_URL = "/static/"
APPEND_SLASH = True

# local development settings
try:
    from .local_settings import *
except ImportError:
    pass

# local config settings
try:
    from settings_model.config import *
except ImportError:
    pass



import os
from setuptools import find_packages, setup

import settings_model


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# get README
with open("README.rst") as f:
    long_description = f.read()

setup(
    name="django-settings-model",
    version=settings_model.__version__,
    packages=find_packages(),
    install_requires=["Django>=2", "pytz"],
    description=(
        "A re-useable Django app for building models that modify Django settings."
    ),
    long_description=long_description,
    url="https://github.com/gregschmit/django-settings-model",
    author="Gregory N. Schmit",
    author_email="me@gregschmit.com",
    license="MIT",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)
